#!/usr/bin/env python
"""
applet.py
by Jason Conti
February 16, 2010

The main applet code.
"""

import pygtk
pygtk.require('2.0')

import gtk
import gnomeapplet
import gobject
import sys

from optparse import OptionParser

class Applet(object):
  """The main applet wrapper."""
  def __init__(self, applet, iid):
    self._applet = applet

    self._button = gtk.ToggleButton("Sample Very Very Long String")
    self._button.connect("button_press_event", self.do_not_eat_button_press)
    self._button.connect("toggled", self.on_button_toggled)
    self._applet.add(self._button)

    self._applet.setup_menu("""
      <popup name="button3">
        <menuitem name="About" verb="About" _label="_About" pixtype="stock" pixname="gtk-about"/>
      </popup>
      """, [
      ("About", lambda a, b: self.on_show_about(a))
      ])

    self._window = MessageWindow()
    self._test_counter = 0

    self._applet.connect("destroy", self.on_applet_destroyed)
    self._applet.show_all()

  def get_orient(self):
    return self._applet.get_orient()

  def get_origin(self):
    return self._button.get_window().get_origin()

  def get_size(self):
    return self._button.get_window().get_size()

  # Callbacks

  def do_not_eat_button_press(self, widget, event):
    """Adapted from the GNOME clock applet."""
    if event.button != 1:
      widget.stop_emission("button_press_event")

    return False

  def on_applet_destroyed(self, *args):
    pass

  def on_button_toggled(self, button, *args):
    if button.get_active():
      self._window.add_message("blah " + str(self._test_counter))
      self._test_counter += 1
      self._window.show_all()
      self._window.align_with_applet(self)
    else:
      self._window.hide_all()

  def on_show_about(self, *args):
    print "About clicked"

class MessageWindow(gtk.Window):
  """Shows the log of notification messages."""
  def __init__(self, limit_messages = 10):
    gtk.Window.__init__(self)

    self._limit_messages = limit_messages

    self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
    self.set_decorated(False)
    self.set_resizable(False)
    self.set_title("Message Window")

    self._vbox = gtk.VBox(True)

    self._messages = []

    self.add(self._vbox)

  def add_message(self, message):
    """Adds a message to the message window."""
    label = gtk.Label(message)
    self._messages.append(label)
    self._vbox.pack_start(label)

    if len(self._messages) > self._limit_messages:
      message = self._messages.pop(0)
      self._vbox.remove(message)

  def align_with_applet(self, applet):
    """Adapted from position_calendar_popup in the GNOME clock applet."""
    x, y = applet.get_origin()
    w, h = self.size_request()
    button_w, button_h = applet.get_size()

    screen = self.get_screen()
    monitor = None

    for i in range(screen.get_n_monitors()):
      m = screen.get_monitor_geometry(i)

      if x >= m.x and x <= m.x + m.width and y >= m.y and y <= m.y + m.height:
        monitor = m
        break

    # Probably xinerama
    if monitor == None:
      monitor = gtk.gdk.Rectangle(0, 0, screen.get_width(), screen.get_height())

    orient = applet.get_orient()

    if orient == gnomeapplet.ORIENT_RIGHT:
      x += button_w
      if (y + h) > monitor.y + monitor.height:
        y -= (y + h) - (monitor.y + monitor.height)

      if (y + h) > (monitor.height / 2):
        gravity = gtk.gdk.GRAVITY_SOUTH_WEST
      else:
        gravity = gtk.gdk.GRAVITY_NORTH_WEST

    elif orient == gnomeapplet.ORIENT_LEFT:
      x -= w
      if (y + h) > monitor.y + monitor.height:
        y -= (y + h) - (monitor.y + monitor.height)

      if (y + h) > (monitor.height / 2):
        gravity = gtk.gdk.GRAVITY_SOUTH_EAST
      else:
        gravity = gtk.gdk.GRAVITY_NORTH_EAST

    elif orient == gnomeapplet.ORIENT_DOWN:
      y += button_h
      if (x + w) > monitor.x + monitor.width:
        x -= (x + w) - (monitor.x + monitor.width)

      gravity = gtk.gdk.GRAVITY_NORTH_WEST

    else:
      y -= h
      if (x + w) > monitor.x + monitor.width:
        x -= (x + w) - (monitor.x + monitor.width)

      gravity = gtk.gdk.GRAVITY_SOUTH_WEST

    self.move(x, y)
    self.set_gravity(gravity)

def applet_factory(applet, iid):
  Applet(applet, iid)
  return True

def run_in_window():
  main_window = gtk.Window()
  main_window.set_title("Applet")
  main_window.connect("destroy", gtk.main_quit)
  app = gnomeapplet.Applet()
  applet_factory(app, None)
  app.reparent(main_window)
  main_window.show_all()
  gtk.main()

def initialize_applet():
  gnomeapplet.bonobo_factory("OAFIID:NotifyLogApplet_Factory", 
      gnomeapplet.Applet.__gtype__, 
      "hello", "0", applet_factory)

# Get command line options
def parse_options(args):
  usage = "Usage: %prog [options]"
  parser = OptionParser(usage=usage)

  parser.add_option("-w", "--windowed", action="store_true", dest="windowed", default=False, metavar=" ",
      help="run the applet in a window for testing purposes [default: %default]")
  # Need to specify these options, because they are provided when adding the applet to a panel
  # and OptionParser raises an exception for invalid args
  parser.add_option("--oaf-activate-iid")
  parser.add_option("--oaf-ior-fd")

  options, extra = parser.parse_args(args)

  return options

def main():
  options = parse_options(sys.argv)

  if options.windowed:
    run_in_window()
  else:
    initialize_applet()

if __name__ == '__main__':
  main()
